@@ -37,7 +37,6 @@
 #include "nautilus-freedesktop-dbus.h"
 #include "nautilus-image-properties-page.h"
 #include "nautilus-previewer.h"
-#include "nautilus-progress-ui-handler.h"
 #include "nautilus-self-check-functions.h"
 #include "nautilus-shell-search-provider.h"
 #include "nautilus-window.h"
@@ -66,15 +65,9 @@
 #include <gio/gio.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-stock-dialogs.h>
-#include <libnotify/notify.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#define NAUTILUS_ACCEL_MAP_SAVE_DELAY 30
-
-/* The saving of the accelerator map was requested  */
-static gboolean save_of_accel_map_requested = FALSE;
-
 G_DEFINE_TYPE (NautilusApplication, nautilus_application, GTK_TYPE_APPLICATION);
 
 struct _NautilusApplicationPriv {
@@ -84,15 +77,41 @@ struct _NautilusApplicationPriv {
 
 	gboolean desktop_override;
 
-	NotifyNotification *unmount_notify;
-
 	NautilusBookmarkList *bookmark_list;
 
 	GtkWidget *connect_server_window;
 
 	NautilusShellSearchProvider *search_provider;
+
+	GList *windows;
 };
 
+void
+nautilus_application_add_accelerator (GApplication *app,
+				      const gchar  *action_name,
+				      const gchar  *accel)
+{
+	const gchar *vaccels[] = {
+		accel,
+		NULL
+	};
+
+	gtk_application_set_accels_for_action (GTK_APPLICATION (app), action_name, vaccels);
+}
+
+GList *
+nautilus_application_get_windows (NautilusApplication *application)
+{
+	return application->priv->windows;
+}
+
+
+NautilusProgressUIHandler *
+nautilus_application_get_progress_ui_handler (NautilusApplication *application)
+{
+	return application->priv->progress_handler;
+}
+
 NautilusBookmarkList *
 nautilus_application_get_bookmarks (NautilusApplication *application)
 {
@@ -113,60 +132,6 @@ nautilus_application_edit_bookmarks (NautilusApplication *application,
 	gtk_window_present (bookmarks_window);
 }
 
-void
-nautilus_application_notify_unmount_done (NautilusApplication *application,
-					  const gchar *message)
-{
-	if (application->priv->unmount_notify) {
-		notify_notification_close (application->priv->unmount_notify, NULL);
-		g_clear_object (&application->priv->unmount_notify);
-	}
-
-	if (message != NULL) {
-		NotifyNotification *unplug;
-		gchar **strings;
-
-		strings = g_strsplit (message, "\n", 0);
-		unplug = notify_notification_new (strings[0], strings[1],
-						  "media-removable");
-		notify_notification_set_hint (unplug,
-					      "desktop-entry", g_variant_new_string ("nautilus"));
-
-		notify_notification_show (unplug, NULL);
-		g_object_unref (unplug);
-		g_strfreev (strings);
-	}
-}
-
-void
-nautilus_application_notify_unmount_show (NautilusApplication *application,
-					  const gchar *message)
-{
-	gchar **strings;
-
-	strings = g_strsplit (message, "\n", 0);
-
-	if (!application->priv->unmount_notify) {
-		application->priv->unmount_notify =
-			notify_notification_new (strings[0], strings[1],
-						 "media-removable");
-
-		notify_notification_set_hint (application->priv->unmount_notify,
-					      "desktop-entry", g_variant_new_string ("nautilus"));
-		notify_notification_set_hint (application->priv->unmount_notify,
-					      "transient", g_variant_new_boolean (TRUE));
-		notify_notification_set_urgency (application->priv->unmount_notify,
-						 NOTIFY_URGENCY_CRITICAL);
-	} else {
-		notify_notification_update (application->priv->unmount_notify,
-					    strings[0], strings[1],
-					    "media-removable");
-	}
-
-	notify_notification_show (application->priv->unmount_notify, NULL);
-	g_strfreev (strings);
-}
-
 static gboolean
 check_required_directories (NautilusApplication *application)
 {
@@ -374,21 +339,6 @@ do_upgrades_once (NautilusApplication *self)
 	g_free (xdg_dir);
 }
 
-static gboolean
-another_navigation_window_already_showing (NautilusApplication *application,
-					   NautilusWindow *the_window)
-{
-	GList *l;
-	
-	for (l = gtk_application_get_windows (GTK_APPLICATION (application)); l; l = l->next) {
-		if (NAUTILUS_IS_WINDOW (l->data) && l->data != the_window) {
-			return TRUE;
-		}
-	}
-	
-	return FALSE;
-}
-
 NautilusWindow *
 nautilus_application_create_window (NautilusApplication *application,
 				    GdkScreen           *screen)
@@ -396,10 +346,12 @@ nautilus_application_create_window (NautilusApplication *application,
 	NautilusWindow *window;
 	char *geometry_string;
 	gboolean maximized;
+	gint n_windows;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	nautilus_profile_start (NULL);
 
+	n_windows = g_list_length (application->priv->windows);
 	window = nautilus_window_new (screen);
 
 	maximized = g_settings_get_boolean
@@ -414,16 +366,16 @@ nautilus_application_create_window (NautilusApplication *application,
 		(nautilus_window_state, NAUTILUS_WINDOW_STATE_GEOMETRY);
 	if (geometry_string != NULL &&
 	    geometry_string[0] != 0) {
-		/* Ignore saved window position if a window with the same
-		 * location is already showing. That way the two windows
-		 * wont appear at the exact same location on the screen.
+		/* Ignore saved window position if another window is already showing.
+		 * That way the two windows wont appear at the exact same
+		 * location on the screen.
 		 */
 		eel_gtk_window_set_initial_geometry_from_string 
 			(GTK_WINDOW (window), 
 			 geometry_string,
 			 NAUTILUS_WINDOW_MIN_WIDTH,
 			 NAUTILUS_WINDOW_MIN_HEIGHT,
-			 another_navigation_window_already_showing (application, window));
+			 n_windows > 0);
 	}
 	g_free (geometry_string);
 
@@ -437,6 +389,7 @@ static NautilusWindowSlot *
 get_window_slot_for_location (NautilusApplication *application, GFile *location)
 {
 	NautilusWindowSlot *slot;
+	NautilusWindow *window;
 	GList *l, *sl;
 
 	slot = NULL;
@@ -447,12 +400,11 @@ get_window_slot_for_location (NautilusApplication *application, GFile *location)
 		g_object_ref (location);
 	}
 
-	for (l = gtk_application_get_windows (GTK_APPLICATION (application)); l; l = l->next) {
-		if (!NAUTILUS_IS_WINDOW (l->data) || NAUTILUS_IS_DESKTOP_WINDOW (l->data))
-			continue;
+	for (l = application->priv->windows; l != NULL; l = l->next) {
+		window = l->data;
 
-		for (sl = nautilus_window_get_slots (NAUTILUS_WINDOW (l->data)); sl; sl = sl->next) {
-			NautilusWindowSlot *current = NAUTILUS_WINDOW_SLOT (sl->data);
+		for (sl = nautilus_window_get_slots (window); sl; sl = sl->next) {
+			NautilusWindowSlot *current = sl->data;
 			GFile *slot_location = nautilus_window_slot_get_location (current);
 
 			if (slot_location && g_file_equal (slot_location, location)) {
@@ -570,51 +522,12 @@ go_to_server_cb (NautilusWindow *window,
 		 gpointer        user_data)
 {
 	gboolean retval;
+	NautilusFile *file;
 
 	if (error == NULL) {
-		GBookmarkFile *bookmarks;
-		GError *error = NULL;
-		char *datadir;
-		char *filename;
-		char *uri;
-		char *title;
-		NautilusFile *file;
-		gboolean safe_to_save = TRUE;
-
 		file = nautilus_file_get_existing (location);
-
-		bookmarks = g_bookmark_file_new ();
-		datadir = g_build_filename (g_get_user_config_dir (), "nautilus", NULL);
-		filename = g_build_filename (datadir, "servers", NULL);
-		g_mkdir_with_parents (datadir, 0700);
-		g_free (datadir);
-		g_bookmark_file_load_from_file (bookmarks,
-						filename,
-						&error);
-		if (error != NULL) {
-			if (! g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
-				/* only warn if the file exists */
-				g_warning ("Unable to open server bookmarks: %s", error->message);
-				safe_to_save = FALSE;
-			}
-			g_error_free (error);
-		}
-
-		if (safe_to_save) {
-			uri = nautilus_file_get_uri (file);
-			title = nautilus_file_get_display_name (file);
-			g_bookmark_file_set_title (bookmarks, uri, title);
-			g_bookmark_file_set_visited (bookmarks, uri, -1);
-			g_bookmark_file_add_application (bookmarks, uri, NULL, NULL);
-			g_free (uri);
-			g_free (title);
-
-			g_bookmark_file_to_file (bookmarks, filename, NULL);
-		}
-
+		nautilus_connect_server_dialog_add_server (file);
 		nautilus_file_unref (file);
-		g_free (filename);
-		g_bookmark_file_free (bookmarks);
 
 		retval = TRUE;
 	} else {
@@ -656,14 +569,12 @@ nautilus_application_connect_server (NautilusApplication *application,
 	if (dialog == NULL) {
 		dialog = nautilus_connect_server_dialog_new (window);
 		g_signal_connect (dialog, "response", G_CALLBACK (on_connect_server_response), application);
-		application->priv->connect_server_window = GTK_WIDGET (dialog);
+		application->priv->connect_server_window = dialog;
 
 		g_object_add_weak_pointer (G_OBJECT (dialog),
 					   (gpointer *) &application->priv->connect_server_window);
 	}
 
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-	gtk_window_set_screen (GTK_WINDOW (dialog), gtk_window_get_screen (GTK_WINDOW (window)));
 	gtk_window_present (GTK_WINDOW (dialog));
 
 	return dialog;
@@ -683,7 +594,7 @@ nautilus_application_finalize (GObject *object)
 	g_clear_object (&application->priv->fdb_manager);
 	g_clear_object (&application->priv->search_provider);
 
-	notify_uninit ();
+	g_list_free (application->priv->windows);
 
         G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
@@ -930,7 +841,9 @@ nautilus_application_handle_local_options (GApplication *application,
 		self->priv->desktop_override = TRUE;
 		g_action_group_activate_action (G_ACTION_GROUP (application),
 						"close-desktop", NULL);
-	}  else if (g_variant_dict_contains (options, "no-default-window")) {
+	}
+
+	if (g_variant_dict_contains (options, "no-default-window")) {
 		/* We want to avoid trigering the activate signal; so no window is created.
 		 * GApplication doesn't call activate if we return a value >= 0.
 		 * Use EXIT_SUCCESS since is >= 0. */
@@ -949,7 +862,6 @@ nautilus_application_handle_local_options (GApplication *application,
 static void
 nautilus_application_activate (GApplication *app)
 {
-	NautilusApplication *self = NAUTILUS_APPLICATION (app);
 	GFile **files;
 
 	DEBUG ("Calling activate");
@@ -972,14 +884,6 @@ nautilus_application_init (NautilusApplication *application)
 	g_application_add_main_option_entries (G_APPLICATION (application), options);
 }
 
-static void
-init_icons_and_styles (void)
-{
-	/* initialize search path for custom icons */
-	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   NAUTILUS_DATADIR G_DIR_SEPARATOR_S "icons");
-}
-
 static void
 nautilus_application_set_desktop_visible (NautilusApplication *self,
 					  gboolean             visible)
@@ -1006,155 +910,10 @@ update_desktop_from_gsettings (NautilusApplication *self)
 static void
 init_desktop (NautilusApplication *self)
 {
-	update_desktop_from_gsettings (self);
 	g_signal_connect_swapped (gnome_background_preferences, "changed::" NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 				  G_CALLBACK (update_desktop_from_gsettings),
 				  self);
-}
-
-static gboolean 
-nautilus_application_save_accel_map (gpointer data)
-{
-	if (save_of_accel_map_requested) {
-		char *accel_map_filename;
-	 	accel_map_filename = nautilus_get_accel_map_file ();
-	 	if (accel_map_filename) {
-	 		gtk_accel_map_save (accel_map_filename);
-	 		g_free (accel_map_filename);
-	 	}
-		save_of_accel_map_requested = FALSE;
-	}
-
-	return FALSE;
-}
-
-static void 
-queue_accel_map_save_callback (GtkAccelMap *object, gchar *accel_path,
-		guint accel_key, GdkModifierType accel_mods,
-		gpointer user_data)
-{
-	if (!save_of_accel_map_requested) {
-		save_of_accel_map_requested = TRUE;
-		g_timeout_add_seconds (NAUTILUS_ACCEL_MAP_SAVE_DELAY, 
-				nautilus_application_save_accel_map, NULL);
-	}
-}
-
-static void
-update_accel_due_to_scripts_migration (gpointer data, const gchar *accel_path, guint accel_key,
-		GdkModifierType accel_mods, gboolean changed)
-{
-	char *old_scripts_location_esc = data;
-	char *old_scripts_pt;
-
-	old_scripts_pt = strstr (accel_path, old_scripts_location_esc);
-
-	if (old_scripts_pt != NULL) {
-		/* There's a mention of the deprecated scripts directory in the accel. Remove it, and
-		 * add a migrated one. */
-		char *tmp;
-		char *tmp2;
-		GString *new_accel_path;
-
-		/* base part of accel */
-		tmp = g_strndup (accel_path, old_scripts_pt - accel_path);
-		new_accel_path = g_string_new (tmp);
-		g_free (tmp);
-
-		/* new script directory, escaped */
-		tmp = nautilus_get_scripts_directory_path ();
-		tmp2 = nautilus_escape_action_name (tmp, "");
-		g_free (tmp);
-		g_string_append (new_accel_path, tmp2);
-		g_free (tmp2);
-
-		/* script path relative to scripts directory */
-		g_string_append (new_accel_path, old_scripts_pt + strlen (old_scripts_location_esc));
-
-		/* exchange entry */
-		gtk_accel_map_change_entry (accel_path, 0, 0, FALSE);
-		gtk_accel_map_change_entry (new_accel_path->str, accel_key, accel_mods, TRUE);
-
-		g_string_free (new_accel_path, TRUE);
-	}
-}
-
-static void
-init_gtk_accels (void)
-{
-	char *accel_map_filename;
-	gboolean performed_migration = FALSE;
-	char *old_scripts_directory_path = NULL;
-
-	accel_map_filename = nautilus_get_accel_map_file ();
-
-	/* If the accel map file doesn't exist, try to migrate from
-	 * former locations. */
-	if (!g_file_test (accel_map_filename, G_FILE_TEST_IS_REGULAR)) {
-		char *old_accel_map_filename;
-		const gchar *override;
-
-		override = g_getenv ("GNOME22_USER_DIR");
-		if (override) {
-			old_accel_map_filename = g_build_filename (override,
-					"accels", "nautilus", NULL);
-			old_scripts_directory_path = g_build_filename (override,
-				       "nautilus-scripts",
-				       NULL);
-		} else {
-			old_accel_map_filename = g_build_filename (g_get_home_dir (),
-					".gnome2", "accels", "nautilus", NULL);
-			old_scripts_directory_path = g_build_filename (g_get_home_dir (),
-								       ".gnome2",
-								       "nautilus-scripts",
-								       NULL);
-		}
-
-		if (g_file_test (old_accel_map_filename, G_FILE_TEST_IS_REGULAR)) {
-			char *parent_dir;
-
-			parent_dir = g_path_get_dirname (accel_map_filename);
-			if (g_mkdir_with_parents (parent_dir, 0700) == 0) {
-				GFile *accel_map_file;
-				GFile *old_accel_map_file;
-
-				accel_map_file = g_file_new_for_path (accel_map_filename);
-				old_accel_map_file = g_file_new_for_path (old_accel_map_filename);
-
-				/* If the move fails, it's safer to not read any accel map
-				 * on startup instead of reading the old one and possibly
-				 * being stuck with it. */
-				performed_migration = g_file_move (old_accel_map_file, accel_map_file, 0, NULL, NULL, NULL, NULL);
-
-				g_object_unref (accel_map_file);
-				g_object_unref (old_accel_map_file);
-			}
-			g_free (parent_dir);
-		}
-
-		g_free (old_accel_map_filename);
-	}
-
-	/* load accelerator map, and register save callback */
-	gtk_accel_map_load (accel_map_filename);
-	g_free (accel_map_filename);
-
-	if (performed_migration) {
-		/* Migrate accels pointing to scripts */
-		char *old_scripts_location_esc;
-
-		old_scripts_location_esc = nautilus_escape_action_name (old_scripts_directory_path, "");
-		gtk_accel_map_foreach (old_scripts_location_esc, update_accel_due_to_scripts_migration);
-		g_free (old_scripts_location_esc);
-		/* save map immediately */
-		save_of_accel_map_requested = TRUE;
-		nautilus_application_save_accel_map (NULL);
-	}
-
-	g_signal_connect (gtk_accel_map_get (), "changed",
-			  G_CALLBACK (queue_accel_map_save_callback), NULL);
-
-	g_free (old_scripts_directory_path);
+	update_desktop_from_gsettings (self);
 }
 
 static void
@@ -1232,10 +991,6 @@ nautilus_application_startup (GApplication *app)
 	/* register property pages */
 	nautilus_image_properties_page_register ();
 
-	/* initialize theming */
-	init_icons_and_styles ();
-	init_gtk_accels ();
-	
 	/* initialize nautilus modules */
 	nautilus_profile_start ("Modules");
 	nautilus_module_setup ();
@@ -1245,7 +1000,6 @@ nautilus_application_startup (GApplication *app)
 	menu_provider_init_callback ();
 	
 	/* Initialize the UI handler singleton for file operations */
-	notify_init (GETTEXT_PACKAGE);
 	self->priv->progress_handler = nautilus_progress_ui_handler_new ();
 
 	/* Check the user's .nautilus directories and post warnings
@@ -1304,9 +1058,6 @@ nautilus_application_quit_mainloop (GApplication *app)
 	DEBUG ("Quitting mainloop");
 
 	nautilus_icon_info_clear_caches ();
- 	nautilus_application_save_accel_map (NULL);
-
-	nautilus_application_notify_unmount_done (NAUTILUS_APPLICATION (app), NULL);
 
 	G_APPLICATION_CLASS (nautilus_application_parent_class)->quit_mainloop (app);
 }
@@ -1319,15 +1070,15 @@ update_dbus_opened_locations (NautilusApplication *app)
 	GList *locations = NULL;
 	gsize locations_size = 0;
 	gchar **locations_array;
+	NautilusWindow *window;
 
 	g_return_if_fail (NAUTILUS_IS_APPLICATION (app));
 
-	for (l = gtk_application_get_windows (GTK_APPLICATION (app)); l; l = l->next) {
-		if (!NAUTILUS_IS_WINDOW (l->data) || NAUTILUS_IS_DESKTOP_WINDOW (l->data))
-			continue;
+	for (l = app->priv->windows; l != NULL; l = l->next) {
+		window = l->data;
 
-		for (sl = nautilus_window_get_slots (NAUTILUS_WINDOW (l->data)); sl; sl = sl->next) {
-			NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (sl->data);
+		for (sl = nautilus_window_get_slots (window); sl; sl = sl->next) {
+			NautilusWindowSlot *slot = sl->data;
 			gchar *uri = nautilus_window_slot_get_location_uri (slot);
 
 			if (uri) {
@@ -1394,30 +1145,35 @@ static void
 nautilus_application_window_added (GtkApplication *app,
 				   GtkWindow *window)
 {
-	/* chain to parent */
+	NautilusApplication *self = NAUTILUS_APPLICATION (app);
+
 	GTK_APPLICATION_CLASS (nautilus_application_parent_class)->window_added (app, window);
 
-	g_signal_connect (window, "slot-added", G_CALLBACK (on_slot_added), app);
-	g_signal_connect (window, "slot-removed", G_CALLBACK (on_slot_removed), app);
+	if (NAUTILUS_IS_WINDOW (window)) {
+		self->priv->windows = g_list_prepend (self->priv->windows, window);
+		g_signal_connect (window, "slot-added", G_CALLBACK (on_slot_added), app);
+		g_signal_connect (window, "slot-removed", G_CALLBACK (on_slot_removed), app);
+	}
 }
 
 static void
 nautilus_application_window_removed (GtkApplication *app,
 				     GtkWindow *window)
 {
-	GList *l;
+	NautilusApplication *self = NAUTILUS_APPLICATION (app);
 
-	/* chain to parent */
 	GTK_APPLICATION_CLASS (nautilus_application_parent_class)->window_removed (app, window);
 
-	/* if this was the last window, close the previewer */
-	for (l = gtk_application_get_windows (GTK_APPLICATION (app)); l && !NAUTILUS_IS_WINDOW (l->data); l = l->next);
-	if (!l) {
-		nautilus_previewer_call_close ();
+	if (NAUTILUS_IS_WINDOW (window)) {
+		self->priv->windows = g_list_remove_all (self->priv->windows, window);
+		g_signal_handlers_disconnect_by_func (window, on_slot_added, app);
+		g_signal_handlers_disconnect_by_func (window, on_slot_removed, app);
 	}
 
-	g_signal_handlers_disconnect_by_func (window, on_slot_added, app);
-	g_signal_handlers_disconnect_by_func (window, on_slot_removed, app);
+	/* if this was the last window, close the previewer */
+	if (g_list_length (self->priv->windows) == 0) {
+		nautilus_previewer_call_close ();
+	}
 }
 
 static void
