@@ -53,8 +53,8 @@ typedef struct {
 } NautilusDesktopWindowClass;
 
 GType                  nautilus_desktop_window_get_type            (void);
-NautilusDesktopWindow *nautilus_desktop_window_new                 (GtkApplication        *application,
-								    GdkScreen             *screen);
+GtkWidget *            nautilus_desktop_window_get                 (void);
+void                   nautilus_desktop_window_ensure              (void);
 gboolean               nautilus_desktop_window_loaded              (NautilusDesktopWindow *window);
 
 #endif /* NAUTILUS_DESKTOP_WINDOW_H */
