@@ -52,9 +52,7 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_USE_TREE_WIDGET "use_tree_view_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET "trash_confirm_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET "trash_delete_checkbutton"
-#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_OPEN_NEW_WINDOW_WIDGET "new_window_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET "hidden_files_checkbutton"
-#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET "treeview_folders_checkbutton"
 
 /* int enums */
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET "preview_image_size_combobox"
@@ -66,13 +64,9 @@ static const char * const default_view_values[] = {
 };
 
 static const char * const zoom_values[] = {
-	"smallest",
-	"smaller",
 	"small",
 	"standard",
 	"large",
-	"larger",
-	"largest",
 	NULL
 };
 
@@ -86,13 +80,6 @@ static const char * const sort_order_values[] = {
 	NULL
 };
 
-static const char * const date_format_values[] = {
-	"locale",
-	"iso",
-	"informal",
-	NULL
-};
-
 static const char * const preview_values[] = {
 	"always",
 	"local-only",
@@ -358,9 +345,11 @@ nautilus_file_management_properties_dialog_setup_icon_caption_page (GtkBuilder *
 		create_icon_caption_combo_box_items (GTK_COMBO_BOX_TEXT (combo_box), columns);
 		gtk_widget_set_sensitive (combo_box, writable);
 
-		g_signal_connect (combo_box, "changed",
-				  G_CALLBACK (icon_captions_changed_callback),
-				  builder);
+		g_signal_connect_data (combo_box, "changed",
+				       G_CALLBACK (icon_captions_changed_callback),
+				       g_object_ref (builder),
+				       (GClosureNotify)g_object_unref,
+				       0);
 	}
 
 	nautilus_column_list_free (columns);
