@@ -14,8 +14,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; see the file COPYING.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * see <http://www.gnu.org/licenses/>.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  *
@@ -62,7 +61,6 @@ typedef struct {
 struct NautilusQueryEditorDetails {
 	GtkWidget *entry;
 	gboolean change_frozen;
-	guint typing_timeout_id;
 
 	GtkWidget *search_current_button;
 	GtkWidget *search_all_button;
@@ -179,11 +177,6 @@ nautilus_query_editor_dispose (GObject *object)
 
 	editor = NAUTILUS_QUERY_EDITOR (object);
 
-	if (editor->details->typing_timeout_id > 0) {
-		g_source_remove (editor->details->typing_timeout_id);
-		editor->details->typing_timeout_id = 0;
-	}
-
 	g_clear_object (&editor->details->query);
 
 	g_list_free_full (editor->details->rows, (GDestroyNotify) row_destroy);
@@ -265,21 +258,6 @@ entry_activate_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 	g_signal_emit (editor, signals[ACTIVATED], 0);
 }
 
-static gboolean
-typing_timeout_cb (gpointer user_data)
-{
-	NautilusQueryEditor *editor;
-
-	editor = NAUTILUS_QUERY_EDITOR (user_data);
-	editor->details->typing_timeout_id = 0;
-
-	nautilus_query_editor_changed (editor);
-
-	return FALSE;
-}
-
-#define TYPING_TIMEOUT 250
-
 static void
 entry_changed_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 {
@@ -287,14 +265,7 @@ entry_changed_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 		return;
 	}
 
-	if (editor->details->typing_timeout_id > 0) {
-		g_source_remove (editor->details->typing_timeout_id);
-	}
-
-	editor->details->typing_timeout_id =
-		g_timeout_add (TYPING_TIMEOUT,
-			       typing_timeout_cb,
-			       editor);
+	nautilus_query_editor_changed (editor);
 }
 
 /* Type */
@@ -516,7 +487,7 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (combo_box));
 		dialog = gtk_dialog_new_with_buttons (_("Select type"),
 						      GTK_WINDOW (toplevel),
-						      0,
+						      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_USE_HEADER_BAR,
 						      _("_Cancel"), GTK_RESPONSE_CANCEL,
 						      _("Select"), GTK_RESPONSE_OK,
 						      NULL);
@@ -526,11 +497,10 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 						GTK_POLICY_AUTOMATIC,
 						GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
-						     GTK_SHADOW_IN);
-		
+
 		gtk_widget_show (scrolled);
-		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), scrolled, TRUE, TRUE, 6);
+		gtk_container_set_border_width (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 0);
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), scrolled, TRUE, TRUE, 0);
 
 		treeview = gtk_tree_view_new ();
 		gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
@@ -963,7 +933,7 @@ setup_widgets (NautilusQueryEditor *editor)
 			  G_CALLBACK (entry_key_press_event_cb), editor);
 	g_signal_connect (editor->details->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), editor);
-	g_signal_connect (editor->details->entry, "changed",
+	g_signal_connect (editor->details->entry, "search-changed",
 			  G_CALLBACK (entry_changed_cb), editor);
 
 	/* create the Current/All Files selector */
