@@ -16,8 +16,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Cosimo Cecchi <cosimoc@redhat.com>
  *
@@ -43,7 +42,7 @@
 	(G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_TOOLBAR, NautilusToolbarClass))
 
 typedef struct _NautilusToolbar NautilusToolbar;
-typedef struct _NautilusToolbarPriv NautilusToolbarPriv;
+typedef struct _NautilusToolbarPrivate NautilusToolbarPrivate;
 typedef struct _NautilusToolbarClass NautilusToolbarClass;
 
 typedef enum {
@@ -52,14 +51,14 @@ typedef enum {
 } NautilusToolbarMode;
 
 struct _NautilusToolbar {
-	GtkFrame parent;
+	GtkHeaderBar parent;
 
 	/* private */
-	NautilusToolbarPriv *priv;
+	NautilusToolbarPrivate *priv;
 };
 
 struct _NautilusToolbarClass {
-	GtkFrameClass parent_class;
+	GtkHeaderBarClass parent_class;
 };
 
 GType nautilus_toolbar_get_type (void);
@@ -73,5 +72,24 @@ void nautilus_toolbar_set_show_main_bar (NautilusToolbar *self,
 					 gboolean show_main_bar);
 void nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
 					       gboolean show_location_entry);
+void nautilus_toolbar_action_menu_add_item (NautilusToolbar *self,
+					    GMenuItem       *item,
+					    const gchar     *section_name);
+void nautilus_toolbar_reset_menus (NautilusToolbar *self);
+
+void nautilus_toolbar_sync_navigation_buttons (NautilusToolbar *self);
+void nautilus_toolbar_view_menu_widget_set_zoom_level (NautilusToolbar *self,
+						       gdouble level);
+void nautilus_toolbar_update_view_mode (NautilusToolbar *self,
+					const gchar *view_mode);
+
+void nautilus_toolbar_show_sort_menu (NautilusToolbar *self);
+void nautilus_toolbar_show_sort_trash_time (NautilusToolbar *self);
+void nautilus_toolbar_show_sort_search_relevance (NautilusToolbar *self);
+void nautilus_toolbar_show_visible_columns (NautilusToolbar *self);
+void nautilus_toolbar_show_stop (NautilusToolbar *self);
+void nautilus_toolbar_show_reload (NautilusToolbar *self);
+void nautilus_toolbar_hide_stop (NautilusToolbar *self);
+void nautilus_toolbar_hide_reload (NautilusToolbar *self);
 
 #endif /* __NAUTILUS_TOOLBAR_H__ */
