@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Ettore Perazzoli
  * 	    Darin Adler <darin@bentspoon.com>
@@ -133,12 +131,6 @@ struct NautilusViewClass {
 
 	/* Function pointers that don't have corresponding signals */
 
-        /* reset_to_defaults is a function pointer that subclasses must 
-         * override to set sort order, zoom level, etc to match default
-         * values. 
-         */
-        void     (* reset_to_defaults)	         (NautilusView *view);
-
 	/* get_backing uri is a function pointer for subclasses to
 	 * override. Subclasses may replace it with a function that
 	 * returns the URI for the location where to create new folders,
@@ -191,13 +183,6 @@ struct NautilusViewClass {
         void    (* bump_zoom_level)      	(NautilusView *view,
 					  	 int zoom_increment);
 
-        /* zoom_to_level is a function pointer that subclasses must override
-         * to set the zoom level of an object to the specified level. */
-        void    (* zoom_to_level) 		(NautilusView *view, 
-        				         NautilusZoomLevel level);
-
-        NautilusZoomLevel (* get_zoom_level)    (NautilusView *view);
-
 	/* restore_default_zoom_level is a function pointer that subclasses must override
          * to restore the zoom level of an object to a default setting. */
         void    (* restore_default_zoom_level) (NautilusView *view);
@@ -229,7 +214,11 @@ struct NautilusViewClass {
          * It is called (at least) whenever the selection changes. If overridden, 
          * subclasses must call parent class's function.
          */
-        void    (* update_menus)         	(NautilusView *view);
+        void    (* update_context_menus)         	(NautilusView *view);
+
+        void    (* update_toolbar_menus)         	(NautilusView *view);
+
+        void    (* update_actions_state)         	(NautilusView *view);
 
 	/* sort_files is a function pointer that subclasses can override
 	 * to provide a sorting order to determine which files should be
@@ -294,9 +283,8 @@ struct NautilusViewClass {
 	void           (* scroll_to_file)	  (NautilusView          *view,
 						   const char            *uri);
 
-        /* Signals used only for keybindings */
-        gboolean (* trash)                         (NautilusView *view);
-        gboolean (* delete)                        (NautilusView *view);
+	NautilusWindow * (*get_window) 	          (NautilusView *view);
+	GActionGroup *   (*get_action_group) 	  (NautilusView *view);
 };
 
 /* GObject support */
@@ -333,7 +321,6 @@ void                nautilus_view_preview_files                    (NautilusView
 void                nautilus_view_start_batching_selection_changes (NautilusView  *view);
 void                nautilus_view_stop_batching_selection_changes  (NautilusView  *view);
 void                nautilus_view_notify_selection_changed         (NautilusView  *view);
-GtkUIManager *      nautilus_view_get_ui_manager                   (NautilusView  *view);
 NautilusDirectory  *nautilus_view_get_model                        (NautilusView  *view);
 NautilusFile       *nautilus_view_get_directory_as_file            (NautilusView  *view);
 void                nautilus_view_pop_up_background_context_menu   (NautilusView  *view,
@@ -360,6 +347,7 @@ void                nautilus_view_remove_subdirectory             (NautilusView
 								   NautilusDirectory*directory);
 
 gboolean            nautilus_view_is_editable                     (NautilusView *view);
+NautilusWindow *    nautilus_view_get_window 			  (NautilusView *view);
 
 /* NautilusView methods */
 const char *      nautilus_view_get_view_id                (NautilusView      *view);
@@ -400,17 +388,27 @@ gboolean          nautilus_view_supports_zooming           (NautilusView      *v
 void              nautilus_view_bump_zoom_level            (NautilusView      *view,
 							    int                zoom_increment);
 void              nautilus_view_zoom_to_level              (NautilusView      *view,
-							    NautilusZoomLevel  level);
+							    gint               level);
 void              nautilus_view_restore_default_zoom_level (NautilusView      *view);
 gboolean          nautilus_view_can_zoom_in                (NautilusView      *view);
 gboolean          nautilus_view_can_zoom_out               (NautilusView      *view);
-NautilusZoomLevel nautilus_view_get_zoom_level             (NautilusView      *view);
 void              nautilus_view_pop_up_location_context_menu (NautilusView    *view,
 							      GdkEventButton  *event,
 							      const char      *location);
+void              nautilus_view_pop_up_pathbar_context_menu (NautilusView    *view,
+							     GdkEventButton  *event,
+							     const char      *location);
 void              nautilus_view_grab_focus                 (NautilusView      *view);
 void              nautilus_view_update_menus               (NautilusView      *view);
 
+void              nautilus_view_update_context_menus       (NautilusView      *view);
+void              nautilus_view_update_toolbar_menus       (NautilusView      *view);
+void              nautilus_view_update_actions_state       (NautilusView      *view);
+
 gboolean          nautilus_view_get_show_hidden_files      (NautilusView      *view);
+void		  nautilus_view_action_show_hidden_files   (NautilusView      *view,
+							    gboolean           show_hidden);
+
+GActionGroup *    nautilus_view_get_action_group           (NautilusView      *view);
 
 #endif /* NAUTILUS_VIEW_H */
